@@ -158,7 +158,13 @@ draw_legend = function(
   if (is.null(legend_args[["bty"]])) legend_args[["bty"]] = "n"
   if (is.null(legend_args[["horiz"]])) legend_args[["horiz"]] = FALSE
   if (is.null(legend_args[["xpd"]])) legend_args[["xpd"]] = NA
-  if (is.null(legend_args[["pt.bg"]])) legend_args[["pt.bg"]] = bg
+  if (is.null(legend_args[["pt.bg"]])) {
+    if (identical(type, "ridge") && isFALSE(gradient)) {
+      legend_args[["pt.bg"]] = sapply(legend_args[["col"]], function(ccol) seq_palette(ccol, n = 2)[2])
+    } else {
+      legend_args[["pt.bg"]] = if (identical(type, "spineplot")) legend_args[["col"]] else bg
+    }
+  }
   if (
     type %in% c("p", "pointrange", "errorbar") &&
     (length(col) == 1 || length(cex) == 1) &&
@@ -166,10 +172,10 @@ draw_legend = function(
   ) {
     legend_args[["pt.cex"]] = cex
   }
-  if (type %in% c("rect", "ribbon", "polygon", "polypath", "boxplot") || isTRUE(gradient)) {
+  if (type %in% c("rect", "ribbon", "polygon", "polypath", "boxplot", "hist", "histogram", "spineplot", "ridge") || isTRUE(gradient)) {
     legend_args[["pch"]] = 22
     if (is.null(legend_args[["pt.cex"]])) legend_args[["pt.cex"]] = 3.5
-    if (is.null(legend_args[["pt.lwd"]]) && (!is.null(type) && !(type %in% c("rect", "polygon", "polypath", "boxplot")))) {
+    if (is.null(legend_args[["pt.lwd"]]) && (!is.null(type) && !(type %in% c("rect", "polygon", "polypath", "boxplot", "ridge")))) {
       legend_args[["pt.lwd"]] = 0
     }
     if (is.null(legend_args[["y.intersp"]])) legend_args[["y.intersp"]] = 1.25
@@ -202,7 +208,8 @@ draw_legend = function(
   
   ## restore inner margin defaults
   ## (in case the plot region/margins were affected by the preceding tinyplot call)
-  if (any(ooma != 0)) {
+  dynmar = isTRUE(.tpar[["dynmar"]])
+  if (any(ooma != 0) && !dynmar) {
     if ( ooma[1] != 0 & omar[1] == par("mgp")[1] + 1*par("cex.lab") ) omar[1] = 5.1
     if ( ooma[2] != 0 & omar[2] == par("mgp")[1] + 1*par("cex.lab") ) omar[2] = 4.1
     if ( ooma[3] == topmar_epsilon & omar[3] != 4.1 ) omar[3] = 4.1
@@ -213,12 +220,14 @@ draw_legend = function(
   par(omd = c(0,1,0,1))
   ooma = par("oma")
   
-  
   ## Legend to outer side (either right or left) of plot
   if (grepl("right!$|left!$", legend_args[["x"]])) {
     
     outer_right = grepl("right!$", legend_args[["x"]])
     
+    # extra bump for spineplot if outer_right legend (to accommodate secondary y-axis)
+    if (identical(type, "spineplot")) lmar[1] = lmar[1] + 1.1
+    
     ## Switch position anchor (we'll adjust relative to the _opposite_ side below)
     if (outer_right) legend_args[["x"]] = gsub("right!$", "left", legend_args[["x"]])
     if (!outer_right) legend_args[["x"]] = gsub("left!$", "right", legend_args[["x"]])
@@ -235,7 +244,22 @@ draw_legend = function(
     }
     par(mar = omar)
     
-    if (isTRUE(new_plot)) plot.new()
+    # if (isTRUE(new_plot)) plot.new()
+    if (isTRUE(new_plot)) {
+      plot.new()
+      # Experimental: For themed + dynamic plots, we need to make sure the
+      # adjusted plot margins for the legend are reinstated (after being
+      # overwritten by the before.plot.new hook.
+      if (dynmar) {
+        omar = par("mar")
+        if (outer_right) {
+          omar[4] = 0
+        } else {
+          omar[2] = par("mgp")[1] + 1*par("cex.lab")
+        }
+        par(mar = omar)
+      }
+    }
     
     legend_args[["horiz"]] = FALSE
     
@@ -281,9 +305,13 @@ draw_legend = function(
     # GM: The legend inset spacing only works _exactly_ if we refresh the plot
     # area. I'm not sure why (and it works properly if we use the same
     # parameters manually while debugging), but this hack seems to work.
-    par(new = TRUE)
+    ## v0.3.0 update: Using (temporary) hook instead of direct par(new = TRUE)
+    ## assignment to play nice with tinytheme logic.
+    oldhook = getHook("before.plot.new")
+    setHook("before.plot.new", function() par(new = TRUE), action = "append")
+    setHook("before.plot.new", function() par(mar = omar), action = "append")
     plot.new()
-    par(new = FALSE)
+    setHook("before.plot.new", oldhook, action = "replace")
     # Finally, set the inset as part of the legend args.
     legend_args[["inset"]] = c(1+inset, 0)
     
@@ -301,7 +329,7 @@ draw_legend = function(
     ## width---will be off the first time.
     if (outer_bottom) {
       omar[1] = par("mgp")[1] + 1*par("cex.lab")
-      if (isTRUE(has_sub)) omar[1] = omar[1] + 1*par("cex.sub")
+      if (isTRUE(has_sub) && (is.null(.tpar[["side.sub"]]) || .tpar[["side.sub"]]==1)) omar[1] = omar[1] + 1*par("cex.sub")
     } else {
       ## For "top!", the logic is slightly different: We don't expand the outer
       ## margin b/c we need the legend to come underneath the main title. So
@@ -311,7 +339,25 @@ draw_legend = function(
     }
     par(mar = omar)
     
-    if (isTRUE(new_plot)) plot.new()
+    # if (isTRUE(new_plot)) plot.new()
+    if (isTRUE(new_plot)) {
+      plot.new()
+      # Experimental: For themed + dynamic plots, we need to make sure the
+      # adjusted plot margins for the legend are reinstated (after being
+      # overwritten by the before.plot.new hook.
+      if (dynmar) {
+        omar = par("mar")
+        if (outer_bottom) {
+          # omar[1] = par("mgp")[1] + 1*par("cex.lab")
+          omar[1] = theme_clean$mgp[1] + 1*par("cex.lab") ## bit of a hack
+          if (isTRUE(has_sub) && (is.null(.tpar[["side.sub"]]) || .tpar[["side.sub"]]==1)) omar[1] = omar[1] + 1*par("cex.sub")
+        } else {
+          ooma[3] = ooma[3] + topmar_epsilon
+          par(oma = ooma)
+        }
+        par(mar = omar)
+      }
+    }
     
     legend_args[["horiz"]] = TRUE
     
@@ -372,9 +418,13 @@ draw_legend = function(
     # GM: The legend inset spacing only works _exactly_ if we refresh the plot
     # area. I'm not sure why (and it works properly if we use the same
     # parameters manually while debugging), but this hack seems to work.
-    par(new = TRUE)
+    ## v0.3.0 update: Using (temporary) hook instead of direct par(new = TRUE)
+    ## assignment to play nice with tinytheme logic.
+    oldhook = getHook("before.plot.new")
+    setHook("before.plot.new", function() par(new = TRUE), action = "append")
+    setHook("before.plot.new", function() par(mar = omar), action = "append") ## experimental dynmar
     plot.new()
-    par(new = FALSE)
+    setHook("before.plot.new", oldhook, action = "replace")
     # Finally, set the inset as part of the legend args.
     legend_args[["inset"]] = c(0, 1+inset)
     
@@ -524,7 +574,7 @@ gradient_legend = function(legend_args, lmar = NULL, outer_right = NULL, outer_b
   
   if (isFALSE(horiz)) {
     labs_idx = !is.na(lgnd_labs)
-    lgnd_labs[labs_idx] = paste0(" ", lgnd_labs[labs_idx])
+    lgnd_labs[labs_idx] = paste0(" ", format(lgnd_labs[labs_idx]))
     lbl_x_anchor = rasterbox[3]
     ttl_x_anchor = rasterbox[1]
     lbl_adj = c(0, 0.5)
